
body {
    background-color:   @bodyBackground;
}

body.notebook_app {
    overflow: hidden;
}

@media (max-width: 767px) {
    // remove bootstrap-responsive's body padding on small screens
    body.notebook_app {
        padding-left: 0px;
        padding-right: 0px;
    }
}

span#notebook_name {
    height: 1em;
    line-height: 1em;
    padding: 3px;
    border: none;
    font-size: 146.5%;
}

div#notebook_panel {
    margin: 0px 0px 0px 0px;
    padding: 0px;
    .box-shadow(0 -1px 10px rgba(0,0,0,.1));
}
div#notebook {
    font-size: @notebook_font_size;
    line-height: @notebook_line_height;
    overflow-y: scroll;
    overflow-x: auto;
    width: 100%;
    /* This spaces the cell away from the edge of the notebook area */
    padding: 1em 0 1em 0;
    margin: 0px;
    border-top: 1px solid @border_color;
    outline: none;
    .border-box-sizing();
}

div.ui-widget-content {
    border: 1px solid @border_color;
    outline: none;
}

pre.dialog {
    background-color: @cell_background;
    border: 1px solid #ddd;
    .corner-all;
    padding: 0.4em;
    padding-left: 2em;
}

p.dialog {
    padding : 0.2em;
}

/* Word-wrap output correctly.  This is the CSS3 spelling, though Firefox seems
   to not honor it correctly.  Webkit browsers (Chrome, rekonq, Safari) do.
 */
pre, code, kbd, samp { white-space: pre-wrap; }

#fonttest {
    font-family: @monoFontFamily;
}

p {
    margin-bottom:0;
}

.end_space {
    height: 200px;
}
