// Our customizations to bootstrap go here.

@textColor:             @black;
@baseFontSize:          13px;
@monoFontFamily:        monospace;  // to allow user to customize their fonts
@navbarHeight:          36px;

code {
  color: @black; // default code color in bootstrap is #d14 (crimson / amaranth)
}

pre {
  // bootstrap has pre defaults that we don't want to inherit.
  // start pre tag defaults based on the surrounding context instead.
  font-size: inherit;
  line-height: inherit;
}

// Our own global variables for all pages go here

