/* CSS for the cell toolbar */

.celltoolbar {
    border: thin solid #CFCFCF;
    border-bottom: none;
    background : #EEE;
    border-radius : 3px 3px 0px 0px;
    width:100%;
    -webkit-box-pack: end;
    height:22px;
    .hbox();
    .reverse();
}

.ctb_hideshow {
    display:none;
    vertical-align:bottom;
    padding-right: 2px;
}

.celltoolbar > div {
   padding-top: 0px;
}


/* ctb_show is added to the ctb_hideshow div to show the cell toolbar.
   Cell toolbars are only shown when the ctb_global_show class is also set.
*/
.ctb_global_show .ctb_show.ctb_hideshow {
    display: block;
}

.ctb_global_show .ctb_show + .input_area,
.ctb_global_show .ctb_show + div.text_cell_input
{
    border-top-right-radius: 0px;
    border-top-left-radius: 0px;
}

.celltoolbar .button_container select {
    margin: 10px;
    margin-top: 1px;
    margin-bottom: 0px;
    padding:0;
    font-size: 87%;
    width:auto;
    display:inline-block;
    height:18px;
    line-height:18px;
    vertical-align:top;
}

.celltoolbar label{
    display:inline-block;
    height:15px;
    line-height:15px;
    vertical-align:top;
}

.celltoolbar label span {
    font-size: 85%;
}

.celltoolbar input[type=checkbox] {
    margin: 0px;
    margin-left: 4px;
    margin-right: 4px;
}


.celltoolbar .ui-button {
    border: none;
    vertical-align:top;
    height:20px;
    min-width:30px;
}
