.notification_widget {
    color: @navbarLinkColor;
    padding: 1px 12px;
    margin: 2px 4px;
    z-index: 10;
    border: 1px solid #ccc;
    border-radius: @baseBorderRadius;
    background: rgba(240, 240, 240, 0.5);

    &.span {
        padding-right:2px;
    }
}
