#menubar {
}

#menubar .navbar-inner {
  min-height: 28px;
  border-top: 1px;
  border-radius: 0px 0px @baseBorderRadius @baseBorderRadius;
}

#menubar .navbar {
  margin-bottom: 8px;
}

.nav-wrapper {
  border-bottom: 1px solid @navbarBorder;
}

#menubar li.dropdown {
  line-height: 12px;
}

i.menu-icon {
  // add padding to account for float-right
  padding-top: 4px;
}

ul#help_menu li a{
    overflow: hidden;
    padding-right: 2.2em;
    i {
        margin-right: -1.2em;
    }
}