#notification_area {
    z-index: 10;
}

.indicator_area {
    color: @navbarLinkColor;
    padding: 4px 3px;
    margin: 0px;
    width: 11px;
    z-index: 10;
    text-align: center;
}

#kernel_indicator {
    // Pull it to the right, outside the container boundary
    margin-right: -16px;
}

.edit_mode_icon:before {
     .icon(@pencil);
}

.command_mode_icon:before {
     .icon(' ');
}

.kernel_idle_icon:before {
     .icon(@circle-blank);
}

.kernel_busy_icon:before {
     .icon(@circle);
}


