div.text_cell {
    padding: 5px 5px 5px 0px;
    .hbox();
}
@media (max-width: 480px) {
    // remove prompt indentation on small screens
    div.text_cell > div.prompt {
        display: none;
    }
}

div.text_cell_render {
    /*font-family: "Helvetica Neue", Arial, Helvetica, Geneva, sans-serif;*/
    outline: none;
    resize: none;
    width:  inherit;
    border-style: none;
    padding: 0.5em 0.5em 0.5em @code_padding;
    color: @textColor;
}

a.anchor-link:link {
  text-decoration: none;
  padding: 0px 20px;
  visibility: hidden;
}

h1,h2,h3,h4,h5,h6 {
    &:hover .anchor-link {
        visibility: visible;
    }
}

div.cell.text_cell.rendered {
    padding: 0px;
}
