.toolbar {
    padding: 0px 10px;
    margin-top: -5px;

    select, label {
        width: auto;
        height: @baseLineHeight + 6px;
        vertical-align:middle;
        margin-right:2px;
        margin-bottom:0px;
        display: inline;
        font-size: 92%;
        margin-left:0.3em;
        margin-right:0.3em;
        padding: 0px;
        padding-top: 3px;
    }
    .btn {
        padding: 2px 8px;
    }
}

.toolbar .btn-group {
  margin-top: 0px;
}

.toolbar-inner {
  border: none !important;
  .box-shadow(none) !important;
}

#maintoolbar {
  margin-bottom: 0px;
}